/***************************************************************************/
/*                                                                         */
/*  File: vector3.h                                                        */
/*                                                                         */
/***************************************************************************/


#ifndef VECTOR3_H
#define VECTOR3_H


class VECTOR3
{
	public:

	float x,y,z; //x,y,z coordinates

	public:

	VECTOR3(): x(0), y(0), z(0){}

	VECTOR3( const float& a, const float& b, const float& c ): x(a), y(b), z(c){}

	//index a component
	//NOTE: returning a reference allows
	//you to assign the indexed element
	float& operator [] ( const long i )
	{
		return *((&x) + i);
	}

	//compare
	const bool operator == ( const VECTOR3& v ) const
	{
		return (v.x==x && v.y==y && v.z==z);
	}

	const bool operator != ( const VECTOR3& v ) const
	{
		return !(v == *this);
	}

	//negate
	const VECTOR3 operator - () const
	{
		return VECTOR3( -x, -y, -z );
	}

	//assign
	const VECTOR3& operator = ( const VECTOR3& v )
	{
		x = v.x;
		y = v.y;
		z = v.z;

		return *this;
	}

	//increment
	const VECTOR3& operator += ( const VECTOR3& v ) 
	{
		x+=v.x;
		y+=v.y;
		z+=v.z;

		return *this;
	} 

	//decrement
	const VECTOR3& operator -= ( const VECTOR3& v ) 
	{
		x-=v.x;
		y-=v.y;
		z-=v.z;

		return *this;
	} 

	//self-multiply
	const VECTOR3& operator *= ( const float& s )
	{

		x*=s;
		y*=s;
		z*=s;

		return *this;
	}

	//self-divide
	const VECTOR3& operator /= ( const float& s )
	{
		const float r = 1 / s;

		x *= r;
		y *= r;
		z *= r;

		return *this;
	}

	//add
	const VECTOR3 operator + ( const VECTOR3& v ) const
	{
		return VECTOR3(x + v.x, y + v.y, z + v.z);
	}

	//subtract
	const VECTOR3 operator - ( const VECTOR3& v ) const
	{
		return VECTOR3(x - v.x, y - v.y, z - v.z);
	}

	//post-multiply by a float
	const VECTOR3 operator * ( const float& s ) const
	{
		return VECTOR3( x*s, y*s, z*s );
	}

	//pre-multiply by a float
	friend inline const VECTOR3 operator * ( const float& s, const VECTOR3& v )
	{
		return v * s;
	}

	//divide
	const VECTOR3 operator / (float s) const
	{
		s = 1/s;

		return VECTOR3( s*x, s*y, s*z );
	}

	//cross product
	const VECTOR3 cross( const VECTOR3& v ) const
	{
		return VECTOR3( y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x );
	}

	//float dot product
	const float dot( const VECTOR3& v ) const
	{
		return x*v.x + y*v.y + z*v.z;
	}

	//length
	const float length() const
	{
		return (float)sqrt( (double)this->dot(*this) );
	}

	//unit VECTOR3
	const VECTOR3 unit() const
	{
		return (*this) / length();
	}

	//make this a unit VECTOR3
	void normalize()
	{
		(*this) /= length();
	}

	//equal within an error e

	const bool nearlyEquals( const VECTOR3& v, const float e ) const
	{
		return fabs(x-v.x)<e && fabs(y-v.y)<e && fabs(z-v.z)<e;
	}

	static float dot( float a, float b, float c,
		   float x, float y, float z)
	{
		return a*x + b*y + c*z;
	}

	static void TransformCoord(VECTOR3 *vout,  VECTOR3*v, float *m )

	{
		vout->x = dot(v->x, v->y, v->z,
					m[0], m[1], m[2]) + m[3];

		vout->y = dot(v->x, v->y, v->z,
					m[4], m[5], m[6]) + m[7];

		vout->z = dot(v->x, v->y, v->z,
					m[8], m[9], m[10]) + m[11];

		//float w = dot(v->x, v->y, v->z,
		//		      m[12], m[13], m[14]) + v[3]*m[15];

}

};

#endif // VECTOR3_H